/*
 * nodequery-api
 * https://github.com/johnnie502/nodequery-api
 *
 * Copyright (c) 2015 Johnnie Zhang <johnnie502@gmail.com>
 * Licensed under the MIT license.
 */

'use strict';

var request = require('request');
var http = require('http');

var API_BASE            = 'https://nodequery.com/api';
var ACCOUNT_END_POINT   = API_BASE + '/account';
var LIST_END_POINT      = API_BASE + '/servers';
var LOADS_END_POINT     = API_BASE + '/loads';

function NodeQueryAPI(apiKey){
    if (!apiKey){
        throw new Error('API Kye is required');
    }

    this.apiKey = apiKey;

    //user http basic authentication 
    //To manually test your API key using the following script:
    //curl https://nodequery.com/api/account -u YOUR_API_KEY:
    this.request = request.defaults({
        auth:{
            username: apiKey,
            password: ''
        },
        json: true
    });
}

module.exports = NodeQueryAPI;

/*
 * Get the information of your account status
The account endpoint will return basic information like name, timezone, server limit, request count and rate limit. The timezone is given as an integer value of the UTC offset in hours.
https://nodequery.com/help/developer-api
@param callback - callback function for when request is complete
- @param {object} error - an error object if there was a problem processing the request
- @param {http.ClientResponse} response - the raw node http.ClientResponse object
- @param {object} body - the JSON-parsed data
*/
NodeQueryAPI.prototype.accountStatus = function accountStatus(callback){
    var self = this;

    self.request.get(ACCOUNT_END_POINT, function(error, response, body){
        self.handleResponse(error, response, body, callback);
    });
}

/*
* Get a list of server with their information
List all servers for the current account. The returned data will be sorted ascending by name and includes basic server and resource usage data. New and unavailable servers will be marked as inactive.
https://nodequery.com/help/developer-api
@param callback - callback function for when request is complete
- @param {object} error - an error object if there was a problem processing the request
- @param {http.ClientResponse} response - the raw node http.ClientResponse object
- @param {object} body - the JSON-parsed data
*/
NodeQueryAPI.prototype.listServers = function listServers(callback){
    var self = this;

    self.request.get(LIST_END_POINT, function(error, response, body){
        self.handleResponse(error, response, body, callback);
    });
}

/*
* Get server detail by server ID
As soon as you provide a valid id to the servers endpoint you will receive detailed information about the server in your account. A new or unavailable server will be marked as inactive.
* https://nodequery.com/help/developer-api
@param serverID - the ID of server you would like to query
@param callback - callback function for when request is complete
- @param {object} error - an error object if there was a problem processing the request
- @param {http.ClientResponse} response - the raw node http.ClientResponse object
- @param {object} body - the JSON-parsed data
*/
NodeQueryAPI.prototype.serverDetails = function serverDetails(serverID, callback){
    var self = this;

    self.request.get(LIST_END_POINT+'/'+serverID, function(error, response, body){
        self.handleResponse(error, response, body, callback);
    });
}

/*
 Get get loads record of a server using its ID
The loads endpoint will always return all objects with information about each interval. If no data was found for an interval it will return 0 for every value. Possible selectors are currently hourly, daily and monthly. Starting August 1st, 2014 we will also offer a yearly load cache.
https://nodequery.com/help/developer-api
@param loadsType - the type of loads, could be hourly, daily, monthly and yearly
@param serverID - the ID of server you would like to query
@param callback - callback function for when request is complete
-@param {object} error - an error object if there was a problem processing the request
- @param {http.ClientResponse} response - the raw node http.ClientResponse object
- @param {object} body - the JSON-parsed data
*/
NodeQueryAPI.prototype.serverLoads = function serverLoads(loadsType, serverID, callback){
    var self = this;

    self.request.get(LOADS_END_POINT+'/'+loadsType+'/'+serverID, function(error, response, body){
        self.handleResponse(error, response, body, callback);
    });
}

/**
Handle the API request responses.
If there is a request error or the response code is not 200
an error will be passed to the callback.
For a successful request the parsed JSON will be passed to the callback.
@param  {Error}      error    Request error.
@param  {Response}   response Response object.
@param  {String}     body     Response body.
@param  {Function} callback   Callback for when the request is complete.
*/
NodeQueryAPI.prototype.handleResponse = function handleResponse(error, response, body, callback) {
    if (error || response.statusCode !== 200) {
        this.handleError(error, response, body, callback);
        return;
    }

    if (typeof callback === 'function') {
        callback(null, body.data);
    }
};

NodeQueryAPI.prototype.handleError = function handleError(error, response, body, callback) {
    if (error) {
        if (typeof callback === 'function') {
            callback(error);
        }
        return;
    }

    if (response.statusCode !== 200) {
        if (typeof callback === 'function') {
            if (body.error && body.error.message) {
                callback(new Error(body.error.message));
            }
            else {
                callback(new Error(http.STATUS_CODES[response.statusCode]));
            }
        }
        return;
    }

    if (typeof callback === 'function') {
        callback();
    }
};
